@@ -47,6 +47,8 @@ extern "C" {
 #define DW_LNE_hi_user                  0xff /* DWARF3 */
 
 /* debug_info tags */
+// this is not a real dwarf named entry, but I wanted to give it
+// a name so it's more obvious and readable that it's just a type of entry
 #define DW_TAG_null_entry               0x00
 #define DW_TAG_array_type               0x01
 #define DW_TAG_class_type               0x02
@@ -59,7 +61,7 @@ extern "C" {
 #define DW_TAG_member                   0x0d
 #define DW_TAG_pointer_type             0x0f
 #define DW_TAG_reference_type           0x10
-#define DW_TAG_compile_unit             0x11 // 
+#define DW_TAG_compile_unit             0x11
 #define DW_TAG_string_type              0x12
 #define DW_TAG_structure_type           0x13
 #define DW_TAG_subroutine_type          0x15
@@ -200,8 +202,8 @@ extern "C" {
 #define DW_AT_description               0x5a
 #define DW_AT_binary_scale              0x5b
 #define DW_AT_decimal_scale             0x5c
-#define DW_AT_small                     0x5d 
-#define DW_AT_decimal_sign              0x5e 
+#define DW_AT_small                     0x5d
+#define DW_AT_decimal_sign              0x5e
 #define DW_AT_digit_count               0x5f
 #define DW_AT_picture_string            0x60
 #define DW_AT_mutable                   0x61
@@ -214,7 +216,7 @@ extern "C" {
 #define DW_AT_recursive                 0x68 // DWARF 3 additions end
 #define DW_AT_signature                 0x69
 #define DW_AT_main_subprogram           0x6a
-#define DW_AT_data_big_offset           0x6b
+#define DW_AT_data_bit_offset           0x6b
 #define DW_AT_const_expr                0x6c
 #define DW_AT_enum_class                0x6d
 #define DW_AT_linkage_name              0x6e
@@ -253,8 +255,23 @@ extern "C" {
 /* <_lo_user ; _hi_user> Interval is reserved for vendor extensions */
 #define DW_AT_lo_user                   0x2000
 // extensions:
+#define DW_AT_MIPS_linkage_name         0x2007 // Same as DWARF4 DW_AT_linkage_name
+#define DW_AT_GNU_call_site_value       0x2111
+#define DW_AT_GNU_call_site_data_value  0x2112
+#define DW_AT_GNU_call_site_target      0x2113
+#define DW_AT_GNU_call_site_target_clobbered   0x2114
+#define DW_AT_GNU_tail_call             0x2115
 #define DW_AT_GNU_all_tail_call_sites   0x2116
-
+#define DW_AT_GNU_all_call_sites        0x2117
+#define DW_AT_GNU_all_source_call_sites 0x2118
+#define DW_AT_GNU_macros                0x2119
+#define DW_AT_GNU_deleted               0x211a
+#define DW_AT_GNU_dwo_name              0x2130
+#define DW_AT_GNU_dwo_id                0x2131
+#define DW_AT_GNU_ranges_base           0x2132
+#define DW_AT_GNU_addr_base             0x2133
+#define DW_AT_GNU_pubnames              0x2134
+#define DW_AT_GNU_pubtypes              0x2135
 #define DW_AT_hi_user                   0x3fff
 
 #define DW_FORM_addr                    0x01
@@ -668,25 +685,42 @@ typedef struct {
 } RBinDwarfAttrDef;
 
 typedef struct {
-	ut64	length;
-	ut8	*data;
+	ut64 length;
+	ut8 *data;
 } RBinDwarfBlock;
 
-typedef struct {
+// http://www.dwarfstd.org/doc/DWARF4.pdf#page=29&zoom=100,0,0
+typedef enum {
+	DW_AT_KIND_ADDRESS,
+	DW_AT_KIND_BLOCK,
+	DW_AT_KIND_CONSTANT,
+	DW_AT_KIND_EXPRLOC,
+	DW_AT_KIND_FLAG,
+	DW_AT_KIND_LINEPTR,
+	DW_AT_KIND_LOCLISTPTR,
+	DW_AT_KIND_MACPTR,
+	DW_AT_KIND_RANGELISTPTR,
+	DW_AT_KIND_REFERENCE,
+	DW_AT_KIND_STRING,
+} RBinDwarfAttrKind;
+
+typedef struct dwarf_attr_kind {
 	ut64 attr_name;
 	ut64 attr_form;
+	RBinDwarfAttrKind kind;
+	/* This is subideal, as dw_form_data can be anything
+	   we could lose information example: encoding signed
+	   2 byte int into ut64 and then interpreting it as st64 TODO*/
 	union {
 		ut64 address;
-		RBinDwarfBlock block;
-		ut64 constant;
+		ut64 uconstant;
+		st64 sconstant;
 		ut8 flag;
-		ut64 data;
-		st64 sdata;
 		ut64 reference;
-		// ut64	offset; // I'll use it for all the new offset forms in DWARF 5
+		RBinDwarfBlock block;
 		struct {
-			const char *content;
 			ut64 offset;
+			const char *content;
 		} string;
 	};
 } RBinDwarfAttrValue;
@@ -715,6 +749,8 @@ typedef struct {
 	ut64	abbrev_code;
 	size_t	count;
 	size_t	capacity;
+	ut64	offset; // important for parsing types
+	bool	has_children; // important for parsing types
 	RBinDwarfAttrValue *attr_values;
 } RBinDwarfDie;
 
@@ -732,6 +768,7 @@ typedef struct {
 	size_t count;
 	size_t capacity;
 	RBinDwarfCompUnit *comp_units;
+	HtUP/*<ut64 offset, DwarfDie *die>*/ *lookup_table;
 } RBinDwarfDebugInfo;
 
 #define	ABBREV_DECL_CAP		8
@@ -797,13 +834,26 @@ typedef struct {
 	size_t file_names_count;
 } RBinDwarfLineHeader;
 
-#define r_bin_dwarf_line_new(o,a,f,l) o->address=a, o->file = strdup (f?f:""), o->line = l, o->column =0,o
+typedef struct r_bin_dwarf_loc_entry_t {
+	ut64 start;
+	ut64 end;
+	RBinDwarfBlock *expression;
+} RBinDwarfLocRange;
+
+typedef struct r_bin_dwarf_loc_list_t {
+	RList/*<RBinDwarfLocRange>*/  *list;
+	ut64 offset;
+} RBinDwarfLocList;
+
+#define r_bin_dwarf_line_new(o,a,f,l) o->address=a, o->file = strdup (r_str_get (f)), o->line = l, o->column =0,o
 
-R_API RBinDwarfDebugAbbrev *r_bin_dwarf_parse_abbrev(RBin *a, int mode);
 R_API RList *r_bin_dwarf_parse_aranges(RBin *a, int mode);
-R_API RBinDwarfDebugInfo *r_bin_dwarf_parse_info(RBinDwarfDebugAbbrev *da, RBin *a, int mode);
 R_API RList *r_bin_dwarf_parse_line(RBin *a, int mode);
-
+R_API RBinDwarfDebugAbbrev *r_bin_dwarf_parse_abbrev(RBin *a, int mode);
+R_API RBinDwarfDebugInfo *r_bin_dwarf_parse_info(RBinDwarfDebugAbbrev *da, RBin *a, int mode);
+R_API HtUP/*<offset, RBinDwarfLocList*>*/  *r_bin_dwarf_parse_loc(RBin *bin, int addr_size);
+R_API void r_bin_dwarf_print_loc(HtUP /*<offset, RBinDwarfLocList*>*/  *loc_table, int addr_size, PrintfCallback print);
+R_API void r_bin_dwarf_free_loc(HtUP /*<offset, RBinDwarfLocList*>*/  *loc_table);
 R_API void r_bin_dwarf_free_debug_info(RBinDwarfDebugInfo *inf);
 R_API void r_bin_dwarf_free_debug_abbrev(RBinDwarfDebugAbbrev *da);
 
