@@ -21,16 +21,18 @@ extern "C" {
 #include <r_util/r_file.h>
 #include <r_vector.h>
 #include <sdb.h>
-#include <sdb/ht_up.h>
+#include <ht_up.h>
 
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #if __UNIX__
+#ifndef __wasi__
 #include <termios.h>
-#include <sys/ioctl.h>
 #include <sys/wait.h>
+#endif
+#include <sys/ioctl.h>
 #include <sys/socket.h>
 #endif
 #if __WINDOWS__
@@ -48,7 +50,6 @@ extern "C" {
 #endif
 
 /* constants */
-#define CONS_MAX_USER 102400
 #define CONS_BUFSZ 0x4f00
 #define STR_IS_NULL(x) (!x || !x[0])
 
@@ -59,6 +60,7 @@ extern "C" {
 #define R_CONS_GREP_WORDS 10
 #define R_CONS_GREP_WORD_SIZE 64
 #define R_CONS_GREP_TOKENS 64
+#define R_CONS_GREP_COUNT 10
 
 R_LIB_VERSION_HEADER(r_cons);
 
@@ -102,78 +104,14 @@ typedef struct r_cons_grep_t {
 	int amp;
 	int zoom;
 	int zoomy; // if set then its scaled unproportionally
-	int neg;
-	int begin;
-	int end;
-	int icase;
+	int neg[R_CONS_GREP_WORDS];
+	int begin[R_CONS_GREP_WORDS];
+	int end[R_CONS_GREP_WORDS];
+	bool icase;
+	bool ascart;
+	bool code;
 } RConsGrep;
 
-#if 0
-// TODO Might be better than using r_cons_pal_get_i
-// And have smaller RConsPrintablePalette and RConsPalette
-enum {
-	R_CONS_PAL_0x00 = 0,
-	R_CONS_PAL_0x7f,
-	R_CONS_PAL_0xff,
-	R_CONS_PAL_ARGS,
-	R_CONS_PAL_BIN,
-	R_CONS_PAL_BTEXT,
-	R_CONS_PAL_CALL,
-	R_CONS_PAL_CJMP,
-	R_CONS_PAL_CMP,
-	R_CONS_PAL_COMMENT,
-	R_CONS_PAL_CREG,
-	R_CONS_PAL_FLAG,
-	R_CONS_PAL_FLINE,
-	R_CONS_PAL_FLOC,
-	R_CONS_PAL_FLOW,
-	R_CONS_PAL_FLOW2,
-	R_CONS_PAL_FNAME,
-	R_CONS_PAL_HELP,
-	R_CONS_PAL_INPUT,
-	R_CONS_PAL_INVALID,
-	R_CONS_PAL_JMP,
-	R_CONS_PAL_LABEL,
-	R_CONS_PAL_MATH,
-	R_CONS_PAL_MOV,
-	R_CONS_PAL_NOP,
-	R_CONS_PAL_NUM,
-	R_CONS_PAL_OFFSET,
-	R_CONS_PAL_OTHER,
-	R_CONS_PAL_POP,
-	R_CONS_PAL_PROMPT,
-	R_CONS_PAL_PUSH,
-	R_CONS_PAL_CRYPTO,
-	R_CONS_PAL_REG,
-	R_CONS_PAL_RESET,
-	R_CONS_PAL_RET,
-	R_CONS_PAL_SWI,
-	R_CONS_PAL_TRAP,
-	R_CONS_PAL_AI_READ,
-	R_CONS_PAL_AI_WRITE,
-	R_CONS_PAL_AI_EXEC,
-	R_CONS_PAL_AI_SEQ,
-	R_CONS_PAL_AI_ASCII,
-	R_CONS_PAL_AI_UNMAP,
-	R_CONS_PAL_GUI_CFLOW,
-	R_CONS_PAL_GUI_DATAOFFSET,
-	R_CONS_PAL_GUI_BACKGROUND,
-	R_CONS_PAL_GUI_ALT_BACKGROUND,
-	R_CONS_PAL_GUI_BORDER,
-	R_CONS_PAL_LINEHL,
-	R_CONS_PAL_GRAPH_BOX,
-	R_CONS_PAL_GRAPH_BOX2,
-	R_CONS_PAL_GRAPH_BOX3,
-	R_CONS_PAL_GRAPH_BOX4,
-	R_CONS_PAL_GRAPH_TRUE,
-	R_CONS_PAL_GRAPH_FALSE,
-	R_CONS_PAL_GRAPH_TRUFAE,
-	R_CONS_PAL_GRAPH_TRACED,
-	R_CONS_PAL_GRAPH_CURRENT,
-	R_CONS_PAL_LAST
-};
-#endif
-
 enum { ALPHA_RESET = 0x00, ALPHA_FG = 0x01, ALPHA_BG = 0x02, ALPHA_FGBG = 0x03 };
 enum { R_CONS_ATTR_BOLD = 1u << 1,
        R_CONS_ATTR_DIM = 1u << 2,
@@ -227,6 +165,7 @@ typedef struct r_cons_palette_t {
 	RColor other;
 	RColor pop;
 	RColor prompt;
+	RColor bgprompt;
 	RColor push;
 	RColor crypto;
 	RColor reg;
@@ -302,6 +241,7 @@ typedef struct r_cons_printable_palette_t {
 	char *other;
 	char *pop;
 	char *prompt;
+	char *bgprompt;
 	char *push;
 	char *crypto;
 	char *reg;
@@ -418,20 +358,20 @@ typedef struct r_cons_canvas_t {
 #define RUNE_CURVE_CORNER_BR "╯"
 #define RUNE_CURVE_CORNER_BL "╰"
 #define RUNE_LONG_LINE_HORIZ "―"
-#define UTF_CIRCLE "\u25EF"
-#define UTF_BLOCK "\u2588"
+#define R_UTF8_CIRCLE "\u25EF"
+#define R_UTF8_BLOCK "\u2588"
 
 // Emoji
-#define UTF8_POLICE_CARS_REVOLVING_LIGHT "🚨"
-#define UTF8_WHITE_HEAVY_CHECK_MARK "✅"
-#define UTF8_SEE_NO_EVIL_MONKEY "🙈"
-#define UTF8_SKULL_AND_CROSSBONES "☠"
-#define UTF8_KEYBOARD "⌨"
-#define UTF8_LEFT_POINTING_MAGNIFYING_GLASS "🔍"
-#define UTF8_DOOR "🚪"
+#define R_UTF8_POLICE_CARS_REVOLVING_LIGHT "🚨"
+#define R_UTF8_WHITE_HEAVY_CHECK_MARK "✅"
+#define R_UTF8_SEE_NO_EVIL_MONKEY "🙈"
+#define R_UTF8_SKULL_AND_CROSSBONES "☠"
+#define R_UTF8_KEYBOARD "⌨"
+#define R_UTF8_LEFT_POINTING_MAGNIFYING_GLASS "🔍"
+#define R_UTF8_DOOR "🚪"
 
 // Variation Selectors
-#define UTF8_VS16 "\xef\xb8\x8f"
+#define R_UTF8_VS16 "\xef\xb8\x8f"
 
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
@@ -446,15 +386,20 @@ typedef enum { COLOR_MODE_DISABLED = 0, COLOR_MODE_16, COLOR_MODE_256, COLOR_MOD
 typedef struct r_cons_context_t {
 	RConsGrep grep;
 	RStack *cons_stack;
-	char *buffer;
-	int buffer_len;
-	int buffer_sz;
-
+	char *buffer; // TODO: replace with RStrBuf
+	size_t buffer_len;
+	size_t buffer_sz;
+	RStrBuf *error; // r_cons_eprintf / r_cons_errstr / r_cons_errmode
+	int errmode;
 	bool breaked;
+	bool was_breaked;
+	bool unbreakable;
 	RStack *break_stack;
 	RConsEvent event_interrupt;
 	void *event_interrupt_data;
 	int cmd_depth;
+	int cmd_str_depth;
+	bool noflush;
 
 	// Used for per-task logging redirection
 	RLogCallback log_callback; // TODO: RList of callbacks
@@ -469,14 +414,30 @@ typedef struct r_cons_context_t {
 	int color_mode;
 	RConsPalette cpal;
 	RConsPrintablePalette pal;
+
+	RList *sorted_lines;
+	RList *unsorted_lines;
+	int sorted_column; // -1
+	bool demo;
+	bool is_html;
+	bool was_html;
+	bool grep_color;
+	bool grep_highlight;
+	bool filter;
+	bool use_tts;
+	bool flush;
 } RConsContext;
 
 #define HUD_BUF_SIZE 512
 
+typedef struct {
+	int x;
+	int y;
+} RConsCursorPos;
+
 typedef struct r_cons_t {
 	RConsContext *context;
 	char *lastline;
-	int is_html;
 	int lines;
 	int rows;
 	int echo; // dump to stdout in realtime
@@ -487,7 +448,8 @@ typedef struct r_cons_t {
 	int fix_rows;
 	int fix_columns;
 	bool break_lines;
-	int noflush;
+	int optimize;
+	// move into Completion
 	bool show_autocomplete_widget;
 	FILE *fdin; // FILE? and then int ??
 	int fdout; // only used in pipe.c :?? remove?
@@ -506,11 +468,13 @@ typedef struct r_cons_t {
 	RConsFunctionKey cb_fkey;
 
 	void *user; // Used by <RCore*>
-#if __UNIX__
+#if __UNIX__ && !__wasi__
 	struct termios term_raw, term_buf;
 #elif __WINDOWS__
 	DWORD term_raw, term_buf, term_xterm;
 	UINT old_cp;
+	bool bCtrl;
+	bool is_arrow;
 #endif
 	RNum *num;
 	/* Pager (like more or less) to use if the output doesn't fit on the
@@ -521,44 +485,30 @@ typedef struct r_cons_t {
 	bool enable_highlight;
 	int null; // if set, does not show anything
 	int mouse;
-	int is_wine;
+	int is_wine; // -1, 0, 1
 	struct r_line_t *line;
 	const char **vline;
 	int refcnt;
 	R_DEPRECATE bool newline;
 	int vtmode;
-	bool flush;
 	bool use_utf8; // use utf8 features
 	bool use_utf8_curvy; // use utf8 curved corners
 	bool dotted_lines;
 	int linesleep;
 	int pagesize;
+	int maxpage;
 	char *break_word;
 	int break_word_len;
-	ut64 timeout;
-	bool grep_color;
-	bool grep_highlight;
-	bool use_tts;
-	bool filter;
+	ut64 timeout; // must come from r_time_now_mono()
 	char* (*rgbstr)(char *str, size_t sz, ut64 addr);
 	bool click_set;
 	int click_x;
 	int click_y;
 	bool show_vals;		// show which section in Vv
 	// TODO: move into instance? + avoid unnecessary copies
+	RConsCursorPos cpos;
 } RCons;
 
-// XXX THIS MUST BE A SINGLETON AND WRAPPED INTO RCons */
-/* XXX : global variables? or a struct with a singleton? */
-//extern FILE *stdin_fd;
-//extern FILE *r_cons_stdin_fd;
-//extern int r_cons_stdout_fd;
-//extern int r_cons_stdout_file;
-//extern char *r_cons_filterline;
-//extern char *r_cons_teefile;
-// not needed anymoar
-//extern int (*r_cons_user_fgets)(char *buf, int len);
-
 #define R_CONS_KEY_F1 0xf1
 #define R_CONS_KEY_F2 0xf2
 #define R_CONS_KEY_F3 0xf3
@@ -574,6 +524,7 @@ typedef struct r_cons_t {
 
 #define R_CONS_KEY_ESC 0x1b
 
+#define R_CONS_CLEAR_FROM_CURSOR_TO_EOL "\x1b[0K\r"
 #define R_CONS_CLEAR_LINE "\x1b[2K\r"
 #define R_CONS_CLEAR_SCREEN "\x1b[2J\r"
 #define R_CONS_CLEAR_FROM_CURSOR_TO_END "\x1b[0J\r"
@@ -767,6 +718,7 @@ typedef struct r_cons_canvas_line_style_t {
 	int color;
 	int symbol;
 	int dot_style;
+	const char *ansicolor;
 } RCanvasLineStyle;
 
 // UTF-8 symbols indexes
@@ -804,10 +756,10 @@ R_API void r_cons_canvas_clear(RConsCanvas *c);
 R_API void r_cons_canvas_print(RConsCanvas *c);
 R_API void r_cons_canvas_print_region(RConsCanvas *c);
 R_API char *r_cons_canvas_to_string(RConsCanvas *c);
-R_API void r_cons_canvas_attr(RConsCanvas *c,const char * attr);
+R_API void r_cons_canvas_attr(RConsCanvas *c,const char *attr);
 R_API void r_cons_canvas_write(RConsCanvas *c, const char *_s);
 R_API bool r_cons_canvas_gotoxy(RConsCanvas *c, int x, int y);
-R_API void r_cons_canvas_goto_write(RConsCanvas *c,int x,int y, const char * s);
+R_API void r_cons_canvas_goto_write(RConsCanvas *c,int x,int y, const char *s);
 R_API void r_cons_canvas_box(RConsCanvas *c, int x, int y, int w, int h, const char *color);
 R_API void r_cons_canvas_circle(RConsCanvas *c, int x, int y, int w, int h, const char *color);
 R_API void r_cons_canvas_line(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
@@ -815,10 +767,11 @@ R_API void r_cons_canvas_line_diagonal(RConsCanvas *c, int x, int y, int x2, int
 R_API void r_cons_canvas_line_square(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
 R_API int r_cons_canvas_resize(RConsCanvas *c, int w, int h);
 R_API void r_cons_canvas_fill(RConsCanvas *c, int x, int y, int w, int h, char ch);
-R_API void r_cons_canvas_line_square_defined (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int bendpoint, int isvert);
-R_API void r_cons_canvas_line_back_edge (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int ybendpoint1, int xbendpoint, int ybendpoint2, int isvert);
+R_API void r_cons_canvas_line_square_defined(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int bendpoint, int isvert);
+R_API void r_cons_canvas_line_back_edge(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int ybendpoint1, int xbendpoint, int ybendpoint2, int isvert);
 R_API RCons *r_cons_new(void);
 R_API RCons *r_cons_singleton(void);
+R_API RConsContext *r_cons_context(void);
 R_API RCons *r_cons_free(void);
 R_API char *r_cons_lastline(int *size);
 R_API char *r_cons_lastline_utf8_ansi_len(int *len);
@@ -827,6 +780,7 @@ R_API bool r_cons_get_click(int *x, int *y);
 
 typedef void (*RConsBreak)(void *);
 R_API bool r_cons_is_breaked(void);
+R_API bool r_cons_was_breaked(void);
 R_API bool r_cons_is_interactive(void);
 R_API bool r_cons_default_context_is_interactive(void);
 R_API void *r_cons_sleep_begin(void);
@@ -849,11 +803,30 @@ R_API int r_cons_is_vtcompat(void);
 R_API void r_cons_w32_clear(void);
 R_API void r_cons_w32_gotoxy(int fd, int x, int y);
 R_API int r_cons_w32_print(const char *ptr, int len, bool vmode);
-R_API int r_cons_win_printf(bool vmode, const char *fmt, ...);
-R_API int r_cons_win_eprintf(bool vmode, const char *fmt, ...);
+R_API int r_cons_win_printf(bool vmode, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
+R_API int r_cons_win_eprintf(bool vmode, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API int r_cons_win_vhprintf(DWORD hdl, bool vmode, const char *fmt, va_list ap);
 #endif
 
+#if 0
+
+Flush Print Buffer
+  0     0     0     null
+  0     0     1     quiet
+  0     1     0     echo
+  0     1     1     buffer
+  1     0     1     flush
+
+#endif
+
+enum {
+	R_CONS_ERRMODE_NULL,   // no buffer no print = null
+	R_CONS_ERRMODE_QUIET,  // buffer no print = quiet
+	R_CONS_ERRMODE_ECHO,   // no buffer, print = like eprintf()
+	R_CONS_ERRMODE_BUFFER, // no buffer, print = like eprintf()
+	R_CONS_ERRMODE_FLUSH,  // no buffer, print = like eprintf + log
+};
+
 R_API void r_cons_push(void);
 R_API void r_cons_pop(void);
 R_API RConsContext *r_cons_context_new(R_NULLABLE RConsContext *parent);
@@ -869,6 +842,11 @@ R_API void r_cons_context_break_pop(RConsContext *context, bool sig);
 R_API char *r_cons_editor(const char *file, const char *str);
 R_API void r_cons_reset(void);
 R_API void r_cons_reset_colors(void);
+R_API char *r_cons_errstr(void);
+R_API void r_cons_errmode(int mode);
+R_API void r_cons_errmodes(const char *mode);
+R_API int r_cons_eprintf(const char *format, ...);
+R_API void r_cons_eflush(void);
 R_API void r_cons_print_clear(void);
 R_API void r_cons_echo(const char *msg);
 R_API void r_cons_zero(void);
@@ -882,10 +860,11 @@ R_API void r_cons_stdout_open(const char *file, int append);
 R_API int  r_cons_stdout_set_fd(int fd);
 R_API void r_cons_gotoxy(int x, int y);
 R_API int r_cons_get_cur_line(void);
+R_API void r_cons_line(int x, int y, int x2, int y2, int ch);
 R_API void r_cons_show_cursor(int cursor);
 R_API char *r_cons_swap_ground(const char *col);
 R_API bool r_cons_drop(int n);
-R_API void r_cons_chop(void);
+R_API void r_cons_chop(void); // XXX R2_580 this function hasnt been implemented and nobody miss it, just rimraf't
 R_API void r_cons_set_raw(bool b);
 R_API void r_cons_set_interactive(bool b);
 R_API void r_cons_set_last_interactive(void);
@@ -893,7 +872,7 @@ R_API void r_cons_set_utf8(bool b);
 R_API void r_cons_grep(const char *grep);
 
 /* output */
-R_API int r_cons_printf(const char *format, ...);
+R_API int r_cons_printf(const char *format, ...) R_PRINTF_CHECK(1, 2);
 R_API void r_cons_printf_list(const char *format, va_list ap);
 R_API void r_cons_strcat(const char *str);
 R_API void r_cons_strcat_at(const char *str, int x, char y, int w, int h);
@@ -901,11 +880,13 @@ R_API void r_cons_strcat_at(const char *str, int x, char y, int w, int h);
 R_API void r_cons_println(const char* str);
 
 R_API void r_cons_strcat_justify(const char *str, int j, char c);
-R_API int r_cons_memcat(const char *str, int len);
+R_API void r_cons_printat(const char *str, int x, char y);
+R_API int r_cons_write(const char *str, int len);
 R_API void r_cons_newline(void);
 R_API void r_cons_filter(void);
 R_API void r_cons_flush(void);
-R_API void r_cons_print_fps (int col);
+// R2_580 - R_API char *r_cons_drain(void);
+R_API void r_cons_print_fps(int col);
 R_API void r_cons_last(void);
 R_API int r_cons_less_str(const char *str, const char *exitkeys);
 R_API void r_cons_less(void);
@@ -914,9 +895,12 @@ R_API void r_cons_memset(char ch, int len);
 R_API void r_cons_visual_flush(void);
 R_API void r_cons_visual_write(char *buffer);
 R_API bool r_cons_is_utf8(void);
-R_API void r_cons_cmd_help(const char * help[], bool use_color);
+R_API bool r_cons_is_windows(void);
+R_API void r_cons_cmd_help(const char *help[], bool use_color);
+R_API void r_cons_cmd_help_json(const char *help[]);
+R_API void r_cons_cmd_help_match(const char *help[], bool use_color, R_BORROW R_NONNULL char *cmd, char spec, bool exact);
 R_API void r_cons_log_stub(const char *output, const char *funcname, const char *filename,
- unsigned int lineno, unsigned int level, const char *tag, const char *fmtstr, ...);
+ unsigned int lineno, unsigned int level, const char *tag, const char *fmtstr, ...) R_PRINTF_CHECK(7, 8);
 
 
 /* input */
@@ -928,6 +912,7 @@ R_API void r_cons_switchbuf(bool active);
 R_API int r_cons_readchar_timeout(ut32 usec);
 R_API int r_cons_any_key(const char *msg);
 R_API int r_cons_eof(void);
+R_API void r_cons_thready(void);
 
 R_API int r_cons_palette_init(const unsigned char *pal);
 R_API int r_cons_pal_set(const char *key, const char *val);
@@ -946,7 +931,7 @@ R_API char *r_cons_rgb_tostring(ut8 r, ut8 g, ut8 b);
 R_API void r_cons_pal_list(int rad, const char *arg);
 R_API void r_cons_pal_show(void);
 R_API int r_cons_get_size(int *rows);
-R_API bool r_cons_isatty(void);
+R_API bool r_cons_is_tty(void);
 R_API int r_cons_get_cursor(int *rows);
 R_API int r_cons_arrow_to_hjkl(int ch);
 R_API char *r_cons_html_filter(const char *ptr, int *newlen);
@@ -956,6 +941,8 @@ R_API void r_cons_rainbow_new(RConsContext *ctx, int sz);
 
 R_API int r_cons_fgets(char *buf, int len, int argc, const char **argv);
 R_API char *r_cons_hud(RList *list, const char *prompt);
+R_API char *r_cons_hud_line(RList *list, const char *prompt);
+R_API char *r_cons_hud_line_string(const char *s);
 R_API char *r_cons_hud_path(const char *path, int dir);
 R_API char *r_cons_hud_string(const char *s);
 R_API char *r_cons_hud_file(const char *f);
@@ -964,8 +951,8 @@ R_API const char *r_cons_get_buffer(void);
 R_API int r_cons_get_buffer_len(void);
 R_API void r_cons_grep_help(void);
 R_API void r_cons_grep_parsecmd(char *cmd, const char *quotestr);
-R_API char * r_cons_grep_strip(char *cmd, const char *quotestr);
-R_API void r_cons_grep_process(char * grep);
+R_API char *r_cons_grep_strip(char *cmd, const char *quotestr);
+R_API void r_cons_grep_process(char *grep);
 R_API int r_cons_grep_line(char *buf, int len); // must be static
 R_API void r_cons_grepbuf(void);
 
@@ -979,7 +966,7 @@ R_API void r_cons_color(int fg, int r, int g, int b);
 
 R_API RColor r_cons_color_random(ut8 alpha);
 R_API void r_cons_invert(int set, int color);
-R_API bool r_cons_yesno(int def, const char *fmt, ...);
+R_API bool r_cons_yesno(int def, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API char *r_cons_input(const char *msg);
 R_API char *r_cons_password(const char *msg);
 R_API bool r_cons_set_cup(bool enable);
@@ -993,6 +980,24 @@ R_API void r_cons_bind(RConsBind *bind);
 R_API const char* r_cons_get_rune(const ut8 ch);
 #endif
 
+/* pixel.c */
+typedef struct {
+	int w;
+	int h;
+	ut8 *buf;
+	size_t buf_size;
+} RConsPixel;
+
+R_API RConsPixel *r_cons_pixel_new(int w, int h);
+R_API void r_cons_pixel_free(RConsPixel *p);
+R_API void r_cons_pixel_flush(RConsPixel *p, int sx, int sy);
+R_API char *r_cons_pixel_drain(RConsPixel *p);
+R_API ut8 r_cons_pixel_get(RConsPixel *p, int x, int y);
+R_API void r_cons_pixel_set(RConsPixel *p, int x, int y, ut8 v);
+R_API void r_cons_pixel_sets(RConsPixel *p, int x, int y, const char *s);
+R_API void r_cons_pixel_fill(RConsPixel *p, int _x, int _y, int w, int h, int v);
+R_API char *r_cons_pixel_tostring(RConsPixel *p);
+
 /* r_line */
 #define R_LINE_BUFSIZE 4096
 #define R_LINE_HISTSIZE 256
@@ -1016,10 +1021,12 @@ typedef struct r_selection_widget_t {
 
 typedef struct r_line_hist_t {
 	char **data;
+	char *match;
 	int size;
 	int index;
 	int top;
 	int autosave;
+	bool do_setup_match;
 } RLineHistory;
 
 typedef struct r_line_buffer_t {
@@ -1067,14 +1074,14 @@ struct r_line_t {
 	// RLineFunctionKeyCb cb_fkey;
 	RConsFunctionKey cb_fkey;
 	/* state , TODO: use more bool */
-	int echo;
-	int has_echo;
+	bool echo;
 	char *prompt;
 	RList/*<str>*/ *kill_ring;
 	int kill_ring_ptr;
 	char *clipboard;
-	int disable;
+	bool disable;
 	void *user;
+	bool histfilter;
 	int (*hist_up)(void *user);
 	int (*hist_down)(void *user);
 	char *contents;
@@ -1088,7 +1095,7 @@ struct r_line_t {
 	RLineHud *hud;
 	RList *sdbshell_hist;
 	RListIter *sdbshell_hist_iter;
-	int vtmode;
+	int vtmode; // R2_580 duplicated and unused from the global RCons.vtmode
 }; /* RLine */
 
 #ifdef R_API
@@ -1099,7 +1106,7 @@ R_API void r_line_free(void);
 R_API char *r_line_get_prompt(void);
 R_API void r_line_set_prompt(const char *prompt);
 R_API int r_line_dietline_init(void);
-R_API void r_line_clipboard_push (const char *str);
+R_API void r_line_clipboard_push(const char *str);
 R_API void r_line_hist_free(void);
 
 typedef int (RLineReadCallback)(void *user, const char *line);
@@ -1108,8 +1115,8 @@ R_API const char *r_line_readline_cb(RLineReadCallback cb, void *user);
 
 R_API int r_line_hist_load(const char *file);
 R_API int r_line_hist_add(const char *line);
-R_API int r_line_hist_save(const char *file);
-R_API int r_line_hist_label(const char *label, void (*cb)(const char*));
+R_API bool r_line_hist_save(const char *file);
+R_API int r_line_hist_label(const char *label, void(*cb)(const char*));
 R_API void r_line_label_show(void);
 R_API int r_line_hist_list(void);
 R_API const char *r_line_hist_get(int n);
@@ -1167,6 +1174,7 @@ typedef enum {
 typedef struct {
 	int x;
 	int y;
+	bool stuck;
 } RPanelsSnow;
 
 typedef struct {
@@ -1190,7 +1198,7 @@ typedef struct r_panels_t {
 	RPanelsMenu *panels_menu;
 	Sdb *db;
 	Sdb *rotate_db;
-	Sdb *almighty_db;
+	Sdb *modal_db;
 	HtPP *mht;
 	RPanelsMode mode;
 	RPanelsFun fun;
