@@ -1,3 +1,5 @@
+/* radare - LGPL - Copyright 2009-2022 - pancake */
+
 #ifndef R2_DEBUG_H
 #define R2_DEBUG_H
 
@@ -9,6 +11,7 @@
 #include <r_reg.h>
 #include <r_egg.h>
 #include <r_bp.h>
+#include <r_cmd.h>
 #include <r_io.h>
 #include <r_syscall.h>
 
@@ -64,7 +67,6 @@ typedef enum {
 	R_DBG_PROC_RAISED = 'R' // has produced a signal, breakpoint, etc..
 } RDebugPidState;
 
-
 // signal handling must support application and debugger level options
 typedef enum {
 	R_DBG_SIGNAL_IGNORE = 0, // ignore signal handler
@@ -73,7 +75,6 @@ typedef enum {
 	//..
 } RDebugSignalMode;
 
-
 /*
  * when a user wants to resume from a breakpoint, we need to know how they want
  * to proceed. these values indicate their intention.
@@ -91,7 +92,6 @@ typedef enum {
 	R_DEBUG_REASON_DEAD = -1,
 	R_DEBUG_REASON_NONE = 0,
 	R_DEBUG_REASON_SIGNAL,
-	R_DEBUG_REASON_SEGFAULT,
 	R_DEBUG_REASON_BREAKPOINT,
 	R_DEBUG_REASON_TRACEPOINT,
 	R_DEBUG_REASON_COND,
@@ -113,10 +113,12 @@ typedef enum {
 	R_DEBUG_REASON_SWI,
 	R_DEBUG_REASON_INT,
 	R_DEBUG_REASON_FPU,
-	R_DEBUG_REASON_USERSUSP,
+	R_DEBUG_REASON_USERSUSP, // ?
+	R_DEBUG_REASON_SEGFAULT,
+	R_DEBUG_REASON_STOPPED,
+	R_DEBUG_REASON_TERMINATED,
 } RDebugReasonType;
 
-
 /* TODO: move to r_anal */
 typedef struct r_debug_frame_t {
 	ut64 addr;
@@ -161,47 +163,42 @@ typedef struct r_debug_desc_t {
 	ut64 off;
 } RDebugDesc;
 
-struct r_debug_snap_diff_t;
-typedef struct r_page_data_t {
-	struct r_debug_snap_diff_t *diff; // Pointing SnapDiff that has this pagedata.
-	ut32 page_off;
-	ut8 *data;
-	ut8 hash[128];
-} RPageData;
-
-struct r_debug_snap_t;
-typedef struct r_debug_snap_diff_t {
-	struct r_debug_snap_t *base;
-	RList *pages; // <RPageData*>
-	RPageData **last_changes; // Last diff entries of each pages
-} RDebugSnapDiff;
-
 typedef struct r_debug_snap_t {
+	char *name;
 	ut64 addr;
 	ut64 addr_end;
-	ut8 *data;
 	ut32 size;
-	ut32 page_num;
-	ut64 timestamp;
-	RHash *hash_ctx;
-	ut8 **hashes; // Hash of each pages
-	RList *history; // <RDebugSnapDiff*>
+	ut8 *data;
 	int perm;
-	char *comment;
+	int user;
+	bool shared;
 } RDebugSnap;
 
-typedef struct r_debug_key {
-	ut64 addr;
-	ut32 id;
-} RDebugKey;
+typedef struct {
+	int cnum;
+	ut64 data;
+} RDebugChangeReg;
+
+typedef struct {
+	int cnum;
+	ut8 data;
+} RDebugChangeMem;
+
+typedef struct r_debug_checkpoint_t {
+	int cnum;
+	RRegArena *arena[R_REG_TYPE_LAST];
+	RList *snaps; // <RDebugSnap>
+} RDebugCheckpoint;
 
 typedef struct r_debug_session_t {
-	RDebugKey key;
-	RListIter *reg[R_REG_TYPE_LAST];
-	RList *memlist; // <RDebugSnapDiff*>
-	/* XXX: DebugSession should have base snapshot of memlist. */
-	//RDebugSnap *base;
-	char *comment;
+	ut32 cnum;
+	ut32 maxcnum;
+	RDebugCheckpoint *cur_chkpt;
+	RVector *checkpoints; /* RVector<RDebugCheckpoint> */
+	HtUP *memory; /* RVector<RDebugChangeMem> */
+	HtUP *registers; /* RVector<RDebugChangeReg> */
+	int reasontype /*RDebugReasonType*/;
+	RBreakpointItem *bp;
 } RDebugSession;
 
 /* Session file format */
@@ -246,6 +243,8 @@ typedef struct r_debug_tracepoint_t {
 } RDebugTracepoint;
 
 typedef struct r_debug_t {
+	// R2_570
+	// use RArchConfig here
 	char *arch;
 	int bits; /// XXX: MUST SET ///
 	int hitinfo;
@@ -312,15 +311,20 @@ typedef struct r_debug_t {
 	RAnal *anal;
 	RList *maps; // <RDebugMap>
 	RList *maps_user; // <RDebugMap>
-	RList *snaps; // <RDebugSnap>
-	RList *sessions; // <RDebugSession>
+
+	bool trace_continue;
+	RAnalOp *cur_op;
+	RDebugSession *session;
+
 	Sdb *sgnls;
-	RCoreBind corebind;
+	RCoreBind coreb;
+	PJ *pj;
 	// internal use only
 	int _mode;
 	RNum *num;
 	REgg *egg;
 	bool verbose;
+	size_t maxsnapsize;
 	bool main_arena_resolved; /* is the main_arena resolved already? */
 	int glibc_version;
 } RDebug;
@@ -357,6 +361,7 @@ typedef struct r_debug_info_t {
 } RDebugInfo;
 
 /* TODO: pass dbg and user data pointer everywhere */
+typedef int (*RDebugCmdCb)(RDebug *dbg, const char *cmd);
 typedef struct r_debug_plugin_t {
 	const char *name;
 	const char *license;
@@ -370,23 +375,23 @@ typedef struct r_debug_plugin_t {
 	/* life */
 	RDebugInfo* (*info)(RDebug *dbg, const char *arg);
 	int (*startv)(int argc, char **argv);
-	int (*attach)(RDebug *dbg, int pid);
-	int (*detach)(RDebug *dbg, int pid);
-	int (*select)(RDebug *dbg, int pid, int tid);
+	bool (*attach)(RDebug *dbg, int pid);
+	bool (*detach)(RDebug *dbg, int pid);
+	bool (*select)(RDebug *dbg, int pid, int tid);
 	RList *(*threads)(RDebug *dbg, int pid);
 	RList *(*pids)(RDebug *dbg, int pid);
 	RList *(*tids)(RDebug *dbg, int pid);
 	RFList (*backtrace)(RDebug *dbg, int count);
 	/* flow */
 	int (*stop)(RDebug *dbg);
-	int (*step)(RDebug *dbg);
-	int (*step_over)(RDebug *dbg);
-	int (*cont)(RDebug *dbg, int pid, int tid, int sig);
-	int (*wait)(RDebug *dbg, int pid);
+	bool (*step)(RDebug *dbg);
+	bool (*step_over)(RDebug *dbg);
+	bool (*cont)(RDebug *dbg, int pid, int tid, int sig);
+	RDebugReasonType (*wait)(RDebug *dbg, int pid);
 	bool (*gcore)(RDebug *dbg, RBuffer *dest);
 	bool (*kill)(RDebug *dbg, int pid, int tid, int sig);
 	RList* (*kill_list)(RDebug *dbg);
-	int (*contsc)(RDebug *dbg, int pid, int sc);
+	bool (*contsc)(RDebug *dbg, int pid, int sc);
 	RList* (*frames)(RDebug *dbg, ut64 at);
 	RBreakpointCallback breakpoint;
 // XXX: specify, pid, tid, or RDebug ?
@@ -400,9 +405,10 @@ typedef struct r_debug_plugin_t {
 	RDebugMap* (*map_alloc)(RDebug *dbg, ut64 addr, int size, bool thp);
 	int (*map_dealloc)(RDebug *dbg, ut64 addr, int size);
 	int (*map_protect)(RDebug *dbg, ut64 addr, int size, int perms);
-	int (*init)(RDebug *dbg);
+	bool (*init)(RDebug *dbg);
 	int (*drx)(RDebug *dbg, int n, ut64 addr, int size, int rwx, int g, int api_type);
 	RDebugDescPlugin desc;
+	RDebugCmdCb cmd;
 	// TODO: use RList here
 } RDebugPlugin;
 
@@ -427,9 +433,9 @@ typedef struct r_debug_pid_t {
  */
 #ifdef R_API
 R_API RDebug *r_debug_new(int hard);
-R_API RDebug *r_debug_free(RDebug *dbg);
+R_API void r_debug_free(RDebug *dbg);
 
-R_API int r_debug_attach(RDebug *dbg, int pid);
+R_API bool r_debug_attach(RDebug *dbg, int pid);
 R_API int r_debug_detach(RDebug *dbg, int pid);
 R_API int r_debug_startv(RDebug *dbg, int argc, char **argv);
 R_API int r_debug_start(RDebug *dbg, const char *cmd);
@@ -441,6 +447,7 @@ R_API const char *r_debug_reason_to_string(int type);
 /* wait for another event */
 R_API RDebugReasonType r_debug_wait(RDebug *dbg, RBreakpointItem **bp);
 
+R_API int r_debug_cmd(RDebug *dbg, const char *s);
 /* continuations */
 R_API int r_debug_step(RDebug *dbg, int steps);
 R_API int r_debug_step_over(RDebug *dbg, int steps);
@@ -471,7 +478,7 @@ R_API bool r_debug_set_arch(RDebug *dbg, const char *arch, int bits);
 R_API bool r_debug_use(RDebug *dbg, const char *str);
 
 R_API RDebugInfo *r_debug_info(RDebug *dbg, const char *arg);
-R_API void r_debug_info_free (RDebugInfo *rdi);
+R_API void r_debug_info_free(RDebugInfo *rdi);
 
 R_API ut64 r_debug_get_baddr(RDebug *dbg, const char *file);
 
@@ -492,7 +499,7 @@ R_API int r_debug_kill_setup(RDebug *dbg, int sig, int action);
 /* handle.c */
 R_API void r_debug_plugin_init(RDebug *dbg);
 R_API int r_debug_plugin_set(RDebug *dbg, const char *str);
-R_API int r_debug_plugin_list(RDebug *dbg, int mode);
+R_API bool r_debug_plugin_list(RDebug *dbg, int mode);
 R_API bool r_debug_plugin_add(RDebug *dbg, RDebugPlugin *foo);
 R_API bool r_debug_plugin_set_reg_profile(RDebug *dbg, const char *str);
 
@@ -502,30 +509,30 @@ R_API RDebugMap *r_debug_map_alloc(RDebug *dbg, ut64 addr, int size, bool thp);
 R_API int r_debug_map_dealloc(RDebug *dbg, RDebugMap *map);
 R_API RList *r_debug_map_list_new(void);
 R_API RDebugMap *r_debug_map_get(RDebug *dbg, ut64 addr);
-R_API RDebugMap *r_debug_map_new (char *name, ut64 addr, ut64 addr_end, int perm, int user);
+R_API RDebugMap *r_debug_map_new(char *name, ut64 addr, ut64 addr_end, int perm, int user);
 R_API void r_debug_map_free(RDebugMap *map);
 R_API void r_debug_map_list(RDebug *dbg, ut64 addr, const char *input);
 R_API void r_debug_map_list_visual(RDebug *dbg, ut64 addr, const char *input, int colors);
 
 /* descriptors */
-R_API RDebugDesc *r_debug_desc_new (int fd, char* path, int perm, int type, int off);
-R_API void r_debug_desc_free (RDebugDesc *p);
+R_API RDebugDesc *r_debug_desc_new(int fd, const char *path, int perm, int type, int off);
+R_API void r_debug_desc_free(RDebugDesc *p);
 R_API int r_debug_desc_open(RDebug *dbg, const char *path);
 R_API int r_debug_desc_close(RDebug *dbg, int fd);
 R_API int r_debug_desc_dup(RDebug *dbg, int fd, int newfd);
 R_API int r_debug_desc_read(RDebug *dbg, int fd, ut64 addr, int len);
 R_API int r_debug_desc_seek(RDebug *dbg, int fd, ut64 addr); // TODO: whence?
 R_API int r_debug_desc_write(RDebug *dbg, int fd, ut64 addr, int len);
-R_API int r_debug_desc_list(RDebug *dbg, int rad);
+R_API int r_debug_desc_list(RDebug *dbg, bool show_commands);
 
 /* registers */
-R_API int r_debug_reg_sync(RDebug *dbg, int type, int write);
-R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, int rad, const char *use_color);
-R_API int r_debug_reg_set(RDebug *dbg, const char *name, ut64 num);
+R_API bool r_debug_reg_sync(RDebug *dbg, int type, int write);
+R_API bool r_debug_reg_list(RDebug *dbg, int type, int size, PJ *pj, int rad, const char *use_color);
+R_API bool r_debug_reg_set(RDebug *dbg, const char *name, ut64 num);
 R_API ut64 r_debug_reg_get(RDebug *dbg, const char *name);
 R_API ut64 r_debug_reg_get_err(RDebug *dbg, const char *name, int *err, utX *value);
 
-R_API ut64 r_debug_execute(RDebug *dbg, const ut8 *buf, int len, int restore);
+R_API bool r_debug_execute(RDebug *dbg, const ut8 *buf, int len, R_OUT ut64 *ret, bool restore, bool ignore_stack);
 R_API bool r_debug_map_sync(RDebug *dbg);
 
 R_API int r_debug_stop(RDebug *dbg);
@@ -536,8 +543,8 @@ R_API RList *r_debug_frames(RDebug *dbg, ut64 at);
 R_API bool r_debug_is_dead(RDebug *dbg);
 R_API int r_debug_map_protect(RDebug *dbg, ut64 addr, int size, int perms);
 /* args XXX: weird food */
-R_API ut64 r_debug_arg_get(RDebug *dbg, int fast, int num);
-R_API bool r_debug_arg_set(RDebug *dbg, int fast, int num, ut64 value);
+R_API ut64 r_debug_arg_get(RDebug *dbg, const char *cc, int num);
+R_API bool r_debug_arg_set(RDebug *dbg, const char *cc, int num, ut64 value);
 
 /* breakpoints (most in r_bp, this calls those) */
 R_API RBreakpointItem *r_debug_bp_add(RDebug *dbg, ut64 addr, int hw, bool watch, int rw, char *module, st64 m_delta);
@@ -568,50 +575,41 @@ R_API int r_debug_drx_unset(RDebug *dbg, int idx);
 
 /* esil */
 R_API ut64 r_debug_num_callback(RNum *userptr, const char *str, int *ok);
-R_API int r_debug_esil_stepi(RDebug *dbg);
+R_API bool r_debug_esil_stepi(RDebug *dbg);
 R_API ut64 r_debug_esil_step(RDebug *dbg, ut32 count);
 R_API ut64 r_debug_esil_continue(RDebug *dbg);
 R_API void r_debug_esil_watch(RDebug *dbg, int rwx, int dev, const char *expr);
 R_API void r_debug_esil_watch_reset(RDebug *dbg);
 R_API void r_debug_esil_watch_list(RDebug *dbg);
-R_API int r_debug_esil_watch_empty(RDebug *dbg);
-R_API void r_debug_esil_prestep (RDebug *d, int p);
+R_API bool r_debug_esil_watch_empty(RDebug *dbg);
+R_API void r_debug_esil_prestep(RDebug *d, int p);
 
-/* snap */
-R_API RDebugSnap *r_debug_snap_new(void);
-R_API void r_debug_snap_free(void *snap);
-R_API int r_debug_snap_delete(RDebug *dbg, int idx);
-R_API void r_debug_snap_list(RDebug *dbg, int idx, int mode);
-R_API int r_debug_snap(RDebug *dbg, ut64 addr);
-R_API int r_debug_snap_comment(RDebug *dbg, int idx, const char *msg);
-R_API RDebugSnapDiff *r_debug_snap_map(RDebug *dbg, RDebugMap *map);
-R_API int r_debug_snap_all(RDebug *dbg, int perms);
-R_API RDebugSnap *r_debug_snap_get(RDebug *dbg, ut64 addr);
-R_API int r_debug_snap_set_idx(RDebug *dbg, int idx);
-R_API int r_debug_snap_set(RDebug *dbg, RDebugSnap *snap);
+/* record & replay */
+// R_API ut8 r_debug_get_byte(RDebug *dbg, ut32 cnum, ut64 addr);
+R_API bool r_debug_add_checkpoint(RDebug *dbg);
+R_API bool r_debug_session_add_reg_change(RDebugSession *session, int arena, ut64 offset, ut64 data);
+R_API bool r_debug_session_add_mem_change(RDebugSession *session, ut64 addr, ut8 data);
+R_API void r_debug_session_restore_reg_mem(RDebug *dbg, ut32 cnum);
+R_API void r_debug_session_list_memory(RDebug *dbg);
+R_API void r_debug_session_serialize(RDebugSession *session, Sdb *db);
+R_API void r_debug_session_deserialize(RDebugSession *session, Sdb *db);
+R_API bool r_debug_session_save(RDebugSession *session, const char *file);
+R_API bool r_debug_session_load(RDebug *dbg, const char *file);
+R_API bool r_debug_trace_ins_before(RDebug *dbg);
+R_API bool r_debug_trace_ins_after(RDebug *dbg);
 
-/* snap diff */
-R_API void r_debug_diff_free(void *p);
-R_API RDebugSnapDiff *r_debug_diff_add(RDebug *dbg, RDebugSnap *base);
-R_API void r_debug_diff_set(RDebug *dbg, RDebugSnapDiff *diff);
-R_API void r_debug_diff_set_base(RDebug *dbg, RDebugSnap *base);
+R_API RDebugSession *r_debug_session_new(void);
+R_API void r_debug_session_free(RDebugSession *session);
 
-/* page data */
-R_API void r_page_data_free(void *p);
+R_API RDebugSnap *r_debug_snap_map(RDebug *dbg, RDebugMap *map);
+R_API bool r_debug_snap_contains(RDebugSnap *snap, ut64 addr);
+R_API ut8 *r_debug_snap_get_hash(RDebugSnap *snap);
+R_API bool r_debug_snap_is_equal(RDebugSnap *a, RDebugSnap *b);
+R_API void r_debug_snap_free(RDebugSnap *snap);
 
-/* debug session */
-R_API void r_debug_session_free(void *p);
-R_API void r_debug_session_list(RDebug *dbg);
-R_API RDebugSession *r_debug_session_add(RDebug *dbg, RListIter **tail);
-R_API bool r_debug_session_delete(RDebug *dbg, int idx);
-R_API bool r_debug_session_comment(RDebug *dbg, int idx, const char *msg);
-R_API void r_debug_session_path(RDebug *dbg, const char *path);
-R_API void r_debug_session_set(RDebug *dbg, RDebugSession *session);
-R_API bool r_debug_session_set_idx(RDebug *dbg, int idx);
-R_API RDebugSession *r_debug_session_get(RDebug *dbg, RListIter *tail);
-R_API void r_debug_session_save(RDebug *dbg, const char *file);
-R_API void r_debug_session_restore(RDebug *dbg, const char *file);
-R_API bool r_debug_step_back(RDebug *dbg);
+R_API int r_debug_step_back(RDebug *dbg, int steps);
+R_API bool r_debug_goto_cnum(RDebug *dbg, ut32 cnum);
+R_API int r_debug_step_cnum(RDebug *dbg, int steps);
 R_API bool r_debug_continue_back(RDebug *dbg);
 
 /* ptrace */
@@ -632,6 +630,7 @@ extern RDebugPlugin r_debug_plugin_rap;
 extern RDebugPlugin r_debug_plugin_gdb;
 extern RDebugPlugin r_debug_plugin_bf;
 extern RDebugPlugin r_debug_plugin_io;
+extern RDebugPlugin r_debug_plugin_winkd;
 extern RDebugPlugin r_debug_plugin_windbg;
 extern RDebugPlugin r_debug_plugin_bochs;
 extern RDebugPlugin r_debug_plugin_qnx;
@@ -643,23 +642,3 @@ extern RDebugPlugin r_debug_plugin_null;
 #endif
 
 #endif
-
-/* regset */
-//R_API struct r_regset_t* r_regset_diff(struct r_regset_t *a, struct r_regset_t *b);
-//R_API int r_regset_set(struct r_regset_t *r, int idx, const char *name, ut64 value);
-//R_API struct r_regset_t *r_regset_new(int size);
-//R_API void r_regset_free(struct r_regset_t *r);
-
-#if 0
-Missing callbacks
-=================
- - alloc
- - dealloc
- - list maps (memory regions)
- - change memory protections
- - touchtrace
- - filedescriptor set/get/mod..
- - get/set signals
- - get regs, set regs
-
-#endif
