@@ -1,7 +1,11 @@
 /* radare - LGPL - Copyright 2010 nibble <.ds@gmail.com> */
 
-#ifndef _INCLUDE_R_FLIST_H_
-#define _INCLUDE_R_FLIST_H_
+#ifndef R2_FLIST_H
+#define R2_FLIST_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #include <stdlib.h>
 #include <string.h>
@@ -11,6 +15,7 @@
 #define RFList void**
 
 #ifdef R_API
+R_API void **r_flist_resize(void **it, int n);
 #define r_flist_rewind(it) while(it!=*it) it--; it++;
 #define r_flist_next(it) *it!=0
 #define r_flist_get(it) *(it++)
@@ -32,8 +37,10 @@ static inline void **r_flist_init(void **it, int n) {
 
 static inline void **r_flist_new(int n) {
 	void **it;
-	if (!(it = (void **)malloc ((n+2) * sizeof (void*))))
+	if (((n + 2) * sizeof(void*)) < sizeof(void*)) return NULL;
+	if (!(it = (void **)calloc ((n + 2), sizeof (void*)))) {
 		return NULL;
+	}
 	return r_flist_init (it, n);
 }
 
@@ -50,7 +57,9 @@ static inline void r_flist_set(void **it, int idx, void *data) {
 static inline void r_flist_delete(void **it, int idx) {
 	r_flist_rewind (it);
 	free (it[idx]);
-	for(it += idx; *it; it++) *it = *(it+1);
+	for (it += idx; *it; it++) {
+		*it = *(it + 1);
+	}
 }
 
 #define r_flist_foreach(it, pos) \
@@ -59,11 +68,24 @@ static inline void r_flist_delete(void **it, int idx) {
 
 static inline void r_flist_free(void **it) {
 	void *pos;
-	r_flist_foreach (it, pos)
+	r_flist_foreach (it, pos) {
 		free (pos);
+	}
 	r_flist_rewind (it);
 	free (--it);
 }
+
+static inline int r_flist_length(void **it) {
+	void *pos;
+	int len = 0;
+	r_flist_foreach (it, pos)
+		len++;
+	return len;
+}
+#endif
+
+#ifdef __cplusplus
+}
 #endif
 
 #endif
