@@ -11,9 +11,19 @@
 #define HAVE_EXPLICIT_MEMSET 0
 #define HAVE_CLOCK_NANOSLEEP 0
 #define HAVE_SIGACTION 1
+#define WANT_THREADS 1
+#define WANT_CAPSTONE 1
+
+#define HAVE_GPERF 0
+#if (HAVE_GPERF) == 1
+#define HAVE_GPERF 0
+#else
+#define HAVE_GPERF 0
+#endif
 
 #ifdef R_MESON_VERSION
 #define R2_PREFIX "/usr/local"
+#define R2_ETCDIR "/etc"
 #define R2_LIBDIR "/usr/local/lib"
 #define R2_INCDIR "/usr/local/include"
 #define R2_DATDIR "/usr/local/share"
@@ -25,12 +35,13 @@
 #define R2_SDB "@SDB@"
 #define R2_ZIGNS "@ZIGNS@"
 #define R2_THEMES "@THEMES@"
-#define R2_FORTUNES "@FORTUNES@"
+#define R2_FORTUNES R_JOIN_3_PATHS ("/usr/local/share", "doc", "radare2")
 #define R2_FLAGS "@FLAGS@"
 #define R2_HUD "@HUD@"
 #else
 #if __WINDOWS__ || _MSC_VER
 #define R2_PREFIX "."
+#define R2_ETCDIR "etc"
 #define R2_LIBDIR "lib"
 #define R2_INCDIR "include\\libr"
 #define R2_DATDIR "share"
@@ -40,6 +51,7 @@
 #define R2_BINDINGS "lib\\bindings"
 #else
 #define R2_PREFIX "/usr/local"
+#define R2_ETCDIR "/etc"
 #define R2_LIBDIR "/usr/local/lib"
 #define R2_INCDIR "/usr/local/include/libr"
 #define R2_DATDIR "/usr/local/share"
@@ -67,11 +79,13 @@
 #define R2_HOME_CACHEDIR    R_JOIN_2_PATHS (".cache", "radare2")
 
 #define R2_HOME_THEMES      R_JOIN_2_PATHS (R2_HOME_DATADIR, "cons")
+#define R2_HOME_FORTUNES    R_JOIN_2_PATHS (R2_HOME_DATADIR, "fortunes")
 #define R2_HOME_PLUGINS     R_JOIN_2_PATHS (R2_HOME_DATADIR, "plugins")
 #define R2_HOME_ZIGNS       R_JOIN_2_PATHS (R2_HOME_DATADIR, "zigns")
 #define R2_HOME_PDB         R_JOIN_2_PATHS (R2_HOME_DATADIR, "pdb")
 #define R2_HOME_PROJECTS    R_JOIN_2_PATHS (R2_HOME_DATADIR, "projects")
 #define R2_HOME_WWWROOT     R_JOIN_2_PATHS (R2_HOME_DATADIR, "www")
+#define R2_HOME_SDB_FCNSIGN R_JOIN_2_PATHS (R2_HOME_DATADIR, "fcnsign")
 #define R2_HOME_SDB_FORMAT  R_JOIN_2_PATHS (R2_HOME_DATADIR, "format")
 #define R2_HOME_HUD         R_JOIN_2_PATHS (R2_HOME_DATADIR, "hud")
 #define R2_HOME_BINRC       R_JOIN_2_PATHS (R2_HOME_DATADIR, "rc.d")
@@ -83,9 +97,9 @@
 #define R2_HOME_RC_DIR      R_JOIN_2_PATHS (R2_HOME_CONFIGDIR, "radare2rc.d")
 #define R2_GLOBAL_RC        R_JOIN_2_PATHS (R2_DATDIR_R2, "radare2rc")
 
-#define HAVE_LIB_MAGIC 1
+#define HAVE_LIB_MAGIC 0
 #define USE_LIB_MAGIC 0
-#define HAVE_LIB_XXHASH 0
+#define HAVE_LIB_XXHASH 1
 #define USE_LIB_XXHASH 0
 
 #ifndef HAVE_LIB_SSL
@@ -94,7 +108,12 @@
 
 #define HAVE_LIBUV 1
 
+#if __MINGW32__
+#define HAVE_PTRACE 0
+#else
 #define HAVE_PTRACE 1
+#endif
+
 #define USE_PTRACE_WRAP 0
 #define HAVE_FORK 1
 #define WANT_DYLINK 1
